package cmis.junit.examples;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;

import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Validates CMIS XML files.
 * 
 * @author Florian Mller
 */
public class CMISValidator {

	private static final String[] SCHEMA_FILES = new String[] { "xml.xsd",
			"CMIS-Core.xsd", "CMIS-Messaging.xsd", "ATOM.xsd", "APP.xsd",
			"CMIS-RestAtom.xsd" };

	/**
	 * Main.
	 */
	public static void main(String[] args) {
		// check args
		if (args.length < 1) {
			System.out.println(CMISValidator.class.getName()
					+ " <schema directory> [file or directory to check]");
			return;
		}

	}

	private Schema fSchema;

	private DocumentBuilder fDocBuilder;

	/**
	 * Constructor.
	 */
	public CMISValidator(String schemaDir) throws Exception {
		// load schema files
		Source[] sources = new Source[SCHEMA_FILES.length];

		for (int i = 0; i < SCHEMA_FILES.length; i++) {
			File schemaFile = new File(schemaDir, SCHEMA_FILES[i]);
			InputStream in = new FileInputStream(schemaFile);
			if (in == null) {
				throw new IllegalArgumentException("Cannot load schema file: "
						+ schemaFile.getAbsolutePath());
			}

			sources[i] = new StreamSource(in);
		}

		// create schema
		fSchema = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI)
				.newSchema(sources);

		// setup document builder
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		dbf.setNamespaceAware(true);
		dbf.setSchema(fSchema);

		fDocBuilder = dbf.newDocumentBuilder();
		fDocBuilder.setErrorHandler(new ErrorHandler() {
			@Override
			public void error(SAXParseException e) throws SAXException {
				ExampleTest.setTestFailed(true);
				System.out.println("Error (line " + e.getLineNumber() + "): "
						+ e.getMessage());
			}

			@Override
			public void fatalError(SAXParseException e) throws SAXException {
				ExampleTest.setTestFailed(true);
				System.out.println("Fatal Error: (line " + e.getLineNumber()
						+ "): " + e.getMessage());
			}

			@Override
			public void warning(SAXParseException e) throws SAXException {
				ExampleTest.setTestFailed(true);
				System.out.println("Warning: (line " + e.getLineNumber()
						+ "): " + e.getMessage());
			}
		});
	}

	/**
	 * Validates a string.
	 */
	public void validate(String fname) {

		File file = new File(fname);

		System.out.println("----------------------------------------------");
		System.out.println("File: " + file.getName());

		try {
			fDocBuilder.parse(file);
			System.out.println("Done.");
		} catch (Exception e) {
			System.out.println("Failed: " + e.getMessage());
		}
	}

}
