package cmis.junit.examples;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.oasis_open.docs.ns.cmis.core._200908.CmisObjectType;
import org.oasis_open.docs.ns.cmis.core._200908.CmisProperty;
import org.oasis_open.docs.ns.cmis.core._200908.CmisTypeDefinitionType;
import org.w3._2005.atom.AtomDateConstruct;
import org.w3._2005.atom.AtomPersonConstruct;
import org.w3._2005.atom.AtomTextConstruct;
import org.w3._2005.atom.Category;
import org.w3._2005.atom.Content;
import org.w3._2005.atom.EntryType;
import org.w3._2005.atom.FeedType;
import org.w3._2005.atom.Id;
import org.w3._2005.atom.Link;
import org.w3._2005.atom.Source;

public class EntryTypeHelper {
	// QNAMEs
	private final static QName _Entry_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "entry");
	private final static QName _Email_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "email");
	private final static QName _Subtitle_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "subtitle");
	private final static QName _Uri_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "uri");
	private final static QName _Title_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "title");
	private final static QName _Author_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "author");
	private final static QName _Name_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "name");
	private final static QName _Contributor_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "contributor");
	private final static QName _Updated_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "updated");
	private final static QName _Feed_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "feed");
	private final static QName _Published_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "published");
	private final static QName _Summary_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "summary");
	private final static QName _Rights_QNAME = new QName(
			"http://www.w3.org/2005/Atom", "rights");

	// CMIS-RestAtom
	private final static QName _Type_QNAME = new QName(
			"http://docs.oasis-open.org/ns/cmis/restatom/200908/", "type");
	private final static QName _Children_QNAME = new QName(
			"http://docs.oasis-open.org/ns/cmis/restatom/200908/", "children");
	private final static QName _RepositoryInfo_QNAME = new QName(
			"http://docs.oasis-open.org/ns/cmis/restatom/200908/",
			"repositoryInfo");
	private final static QName _Object_QNAME = new QName(
			"http://docs.oasis-open.org/ns/cmis/restatom/200908/", "object");
	private final static QName _Uritemplate_QNAME = new QName(
			"http://docs.oasis-open.org/ns/cmis/restatom/200908/",
			"uritemplate");

	private final org.w3._2005.atom.ObjectFactory atomFactory = new org.w3._2005.atom.ObjectFactory();
	private final org.w3._2007.app.ObjectFactory appFactory = new org.w3._2007.app.ObjectFactory();
	private final org.oasis_open.docs.ns.cmis.restatom._200908.ObjectFactory raFactory = new org.oasis_open.docs.ns.cmis.restatom._200908.ObjectFactory();

	// arrays
	private List<AtomPersonConstruct> _authors;
	private List<Category> _categories;
	private List<AtomPersonConstruct> _contributor;
	private List<Link> _links;
	private List<AtomTextConstruct> _rights;
	private List<Object> _anyOther = null;
	private List<Object> _anyLocal = null;

	// scalars
	private Content _content = null;
	private Id _id = null;
	private AtomDateConstruct _published = null;
	private Source _source = null;
	private AtomTextConstruct _summary = null;
	private AtomTextConstruct _title = null;
	private AtomDateConstruct _updated = null;
	private FeedType _children = null;
	private CmisObjectType _object = null;
	private CmisTypeDefinitionType _type = null;

	// backing object
	private EntryType _entry = null;

	// see if list was accessed
	private boolean dirtyAuthors = false;
	private boolean dirtyCategories = false;
	private boolean dirtyContributor = false;
	private boolean dirtyLinks = false;
	private boolean dirtyRights = false;
	private boolean dirtyAnyOther = false;
	private boolean dirtyAnyLocal = false;

	// orig lists
	private HashMap<AtomPersonConstruct, JAXBElement<AtomPersonConstruct>> origAuthors = null;
	private HashMap<Category, Category> origCategories = null;
	private HashMap<AtomPersonConstruct, JAXBElement<AtomPersonConstruct>> origContributors = null;
	private HashMap<Link, Link> origLinks = null;
	private HashMap<AtomTextConstruct, JAXBElement<AtomTextConstruct>> origRights = null;
	private HashMap<Object, Object> origAnyOther = null;

	// orig scalars
	private Object origPublished = null;
	private Object origSummary = null;
	private Object origTitle = null;
	private Object origUpdated = null;
	private Object origChildren = null;
	private Object origObject = null;
	private Object origType = null;

	private boolean dirty = false;

	@SuppressWarnings("unused")
	private EntryTypeHelper() {

	}

	public EntryTypeHelper(EntryType e) {
		if (e == null)
			throw new RuntimeException("EntryType is null");

		// save it
		_entry = e;

		// get new lists
		_authors = new ArrayList<AtomPersonConstruct>();
		_categories = new ArrayList<Category>();
		_contributor = new ArrayList<AtomPersonConstruct>();
		_links = new ArrayList<Link>();
		_rights = new ArrayList<AtomTextConstruct>();
		_anyOther = new ArrayList<Object>();
		_anyLocal = e.getAnyLocal();

		// new hashmaps for orig values
		origAuthors = new HashMap<AtomPersonConstruct, JAXBElement<AtomPersonConstruct>>();
		origCategories = new HashMap<Category, Category>();
		origContributors = new HashMap<AtomPersonConstruct, JAXBElement<AtomPersonConstruct>>();
		origLinks = new HashMap<Link, Link>();
		origRights = new HashMap<AtomTextConstruct, JAXBElement<AtomTextConstruct>>();
		origAnyOther = new HashMap<Object, Object>();

		init(e);
	}

	/**
	 * @return the anyLocal
	 */
	public List<Object> getAnyLocal() {
		dirty = true;
		dirtyAnyLocal = true;
		return _anyLocal;
	}

	/**
	 * @return the anyOther
	 */
	public List<Object> getAnyOther() {
		dirty = true;
		dirtyAnyOther = true;
		return _anyOther;
	}

	/**
	 * @return the authors
	 */
	public List<AtomPersonConstruct> getAuthors() {
		dirty = true;
		dirtyAuthors = true;
		return _authors;
	}

	/**
	 * @return the categories
	 */
	public List<Category> getCategories() {
		dirty = true;
		dirtyCategories = true;
		return _categories;
	}

	/**
	 * @return the children
	 */
	public FeedType getChildren() {
		dirty = true;
		return _children;
	}

	/**
	 * @return the content
	 */
	public Content getContent() {
		dirty = true;

		return _content;
	}

	/**
	 * @return the contributor
	 */
	public List<AtomPersonConstruct> getContributor() {

		return _contributor;
	}

	public EntryType getEntryType() {
		if (!dirty)
			return _entry;

		EntryType e = _entry;

		Iterator it;

		if (dirtyAuthors) {
			dirtyAuthors = false;

			it = _authors.iterator();
			while (it.hasNext()) {
				AtomPersonConstruct apc = (AtomPersonConstruct) it.next();
				// old
				if (origAuthors.containsKey(apc)) {
					origAuthors.remove(apc);
				} else { // new
					e.getItems().add(atomFactory.createAuthor(apc));

				}
			}

			it = origAuthors.values().iterator();
			while (it.hasNext()) {
				e.getItems().remove(it.next());
			}

		}
		// clear the maps
		_authors.clear();
		origAuthors.clear();

		if (dirtyCategories) {
			dirtyCategories = false;

			it = _categories.iterator();
			while (it.hasNext()) {
				Category c = (Category) it.next();
				// old
				if (origCategories.containsKey(c)) {
					origCategories.remove(c);
				} else { // new
					e.getItems().add(c);

				}
			}

			it = origCategories.values().iterator();
			while (it.hasNext()) {
				e.getItems().remove(it.next());
			}

		}
		// clear the maps
		_categories.clear();
		origCategories.clear();

		if (dirtyContributor) {
			dirtyContributor = false;

			it = _contributor.iterator();
			while (it.hasNext()) {
				AtomPersonConstruct apc = (AtomPersonConstruct) it.next();
				// old
				if (origContributors.containsKey(apc)) {
					origContributors.remove(apc);
				} else { // new
					e.getItems().add(atomFactory.createContributor(apc));

				}
			}

			it = origContributors.values().iterator();
			while (it.hasNext()) {
				e.getItems().remove(it.next());
			}

		}
		// clear the maps
		_contributor.clear();
		origContributors.clear();

		if (dirtyLinks) {
			dirtyLinks = false;

			it = _links.iterator();
			while (it.hasNext()) {
				Link l = (Link) it.next();
				// old
				if (origLinks.containsKey(l)) {
					origLinks.remove(l);
				} else { // new
					e.getItems().add(l);
				}
			}

			it = origLinks.values().iterator();
			while (it.hasNext()) {
				e.getItems().remove(it.next());
			}

		}
		// clear the maps
		_links.clear();
		origLinks.clear();

		if (dirtyRights) {
			dirtyRights = false;

			it = _rights.iterator();
			while (it.hasNext()) {
				AtomTextConstruct atc = (AtomTextConstruct) it.next();
				// old
				if (origRights.containsKey(atc)) {
					origRights.remove(atc);
				} else { // new
					e.getItems().add(atomFactory.createRights(atc));
				}
			}

			it = origRights.values().iterator();
			while (it.hasNext()) {
				e.getItems().remove(it.next());
			}

		}
		// clear the maps
		_rights.clear();
		origRights.clear();

		if (dirtyAnyOther) {
			dirtyAnyOther = false;

			it = _anyOther.iterator();
			while (it.hasNext()) {
				Object o = it.next();
				// old
				if (origAnyOther.containsKey(o)) {
					origAnyOther.remove(o);
				} else { // new
					e.getItems().add(o);
				}
			}

			it = origAnyOther.values().iterator();
			while (it.hasNext()) {
				e.getItems().remove(it.next());
			}

		}
		// clear the maps
		_anyOther.clear();
		origAnyOther.clear();

		// redo the internals
		init(e);

		return (e);
	}

	/**
	 * @return the id
	 */
	public Id getId() {
		return _id;
	}

	/**
	 * @return the links
	 */
	public List<Link> getLinks() {
		dirtyLinks = true;
		return _links;
	}

	/**
	 * @return the object
	 */
	public CmisObjectType getObject() {
		return _object;
	}

	/**
	 * @return the published
	 */
	public AtomDateConstruct getPublished() {
		return _published;
	}

	/**
	 * @return the rights
	 */
	public List<AtomTextConstruct> getRights() {
		dirtyRights = true;

		return _rights;
	}

	/**
	 * @return the source
	 */
	public Source getSource() {
		return _source;
	}

	/**
	 * @return the summary
	 */
	public AtomTextConstruct getSummary() {
		return _summary;
	}

	/**
	 * @return the title
	 */
	public AtomTextConstruct getTitle() {
		return _title;
	}

	/**
	 * @return the type
	 */
	public CmisTypeDefinitionType getType() {
		return _type;
	}

	/**
	 * @return the updated
	 */
	public AtomDateConstruct getUpdated() {
		return _updated;
	}

	/**
	 * @param e
	 */
	private void init(EntryType e) {

		_authors.clear();
		_categories.clear();
		_contributor.clear();
		_links.clear();
		_rights.clear();
		_anyOther.clear();

		Iterator<Object> it = e.getItems().iterator();
		while (it.hasNext()) {
			Object o = it.next();

			if (o instanceof Category) {
				Category c = (Category) o;
				_categories.add(c);
				origCategories.put(c, c);
			} else if (o instanceof Content) {
				Content c = (Content) o;
				_content = c;
			} else if (o instanceof Id) {
				Id id = (Id) o;
				_id = id;
			} else if (o instanceof Link) {
				Link l = (Link) o;
				_links.add(l);
				origLinks.put(l, l);
			} else if (o instanceof Source) {
				Source s = (Source) o;
				_source = s;
			} else if (o instanceof JAXBElement<?>) {
				JAXBElement j = (JAXBElement) o;

				if (j.getName().equals(_Title_QNAME)) {
					_title = (AtomTextConstruct) j.getValue();
					origTitle = j;
				} else if (j.getName().equals(_Author_QNAME)) {
					origAuthors.put((AtomPersonConstruct) j.getValue(),
							(JAXBElement<AtomPersonConstruct>) j);
					_authors.add((AtomPersonConstruct) j.getValue());
				} else if (j.getName().equals(_Summary_QNAME)) {
					_summary = (AtomTextConstruct) j.getValue();
					origSummary = j;
				} else if (j.getName().equals(_Contributor_QNAME)) {
					origContributors.put((AtomPersonConstruct) j.getValue(),
							(JAXBElement<AtomPersonConstruct>) j);
					_contributor.add((AtomPersonConstruct) j.getValue());

				} else if (j.getName().equals(_Published_QNAME)) {
					_published = (AtomDateConstruct) j.getValue();
					origPublished = j;
				} else if (j.getName().equals(_Updated_QNAME)) {
					_updated = (AtomDateConstruct) j.getValue();
					origUpdated = j;
				} else if (j.getName().equals(_Rights_QNAME)) {
					origRights.put((AtomTextConstruct) j.getValue(),
							(JAXBElement<AtomTextConstruct>) j);
					_rights.add((AtomTextConstruct) j.getValue());
				} else {
					// should not be here
					System.out.println("Not found JAXBElement of "
							+ j.getValue().getClass().getName());
				}
			}
		}

		// loop through anyOther
		it = e.getAnyOther().iterator();
		while (it.hasNext()) {
			Object o = it.next();

			if (o instanceof JAXBElement<?>) {
				JAXBElement j = (JAXBElement) o;

				if (j.getName().equals(_Children_QNAME)) {
					_children = (FeedType) j.getValue();
				} else if (j.getName().equals(_Type_QNAME)) {
					_type = (CmisTypeDefinitionType) j.getValue();
				} else if (j.getName().equals(_Object_QNAME)) {
					_object = (CmisObjectType) j.getValue();
				} else {
					System.out.println("Not found JAXBElement of "
							+ j.getValue().getClass().getName() + " for "
							+ j.getName().toString());
					_anyOther.add(o);
				}
			} else {
				System.out
						.println("Not found object " + o.getClass().getName());
				_anyOther.add(o);
			}
		}

		// reset the flags
		dirty = false;
		dirtyAuthors = false;
		dirtyCategories = false;
		dirtyContributor = false;
		dirtyLinks = false;
		dirtyRights = false;
		dirtyAnyOther = false;
		dirtyAnyLocal = false;
	}

	public void removeProperty(String propertyDefinitionId) {
		if (propertyDefinitionId == null)
			return;

		Iterator<CmisProperty> it = _object.getProperties().getProperty()
				.iterator();
		while (it.hasNext()) {
			CmisProperty p = it.next();

			if (p.getPropertyDefinitionId().equals(propertyDefinitionId)) {
				this.getObject().getProperties().getProperty().remove(p);
				break;
			}
		}
	}

	/**
	 * @param children
	 *            the children to set
	 */
	public void setChildren(FeedType children) {
		_children = children;

		_entry.getItems().remove(origChildren);
		origChildren = raFactory.createChildren(children);
		_entry.getItems().add(origChildren);
	}

	/**
	 * @param content
	 *            the content to set
	 */
	public void setContent(Content content) {
		_entry.getItems().remove(_content);
		_content = content;
		_entry.getItems().add(_content);

	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setId(Id id) {
		_entry.getItems().remove(_id);
		_id = id;
		_entry.getItems().add(_id);
	}

	/**
	 * @param object
	 *            the object to set
	 */
	public void setObject(CmisObjectType object) {
		_object = object;

		_entry.getItems().remove(origObject);
		origObject = raFactory.createObject(object);
		_entry.getItems().add(origObject);
	}

	/**
	 * @param published
	 *            the published to set
	 */
	public void setPublished(AtomDateConstruct published) {
		_published = published;

		_entry.getItems().remove(origPublished);
		origPublished = atomFactory.createPublished(published);
		_entry.getItems().add(origPublished);
	}

	/**
	 * @param summary
	 *            the summary to set
	 */
	public void setSummary(AtomTextConstruct summary) {
		_summary = summary;

		_entry.getItems().remove(origSummary);
		origSummary = atomFactory.createSummary(summary);
		_entry.getItems().add(origSummary);
	}

	/**
	 * @param title
	 *            the title to set
	 */
	public void setTitle(AtomTextConstruct title) {
		_title = title;

		_entry.getItems().remove(origTitle);
		origTitle = atomFactory.createTitle(title);
		_entry.getItems().add(origTitle);

	}

	/**
	 * @param type
	 *            the type to set
	 */
	public void setType(CmisTypeDefinitionType type) {
		_type = type;

		_entry.getItems().remove(origType);
		origType = raFactory.createType(type);
		_entry.getItems().add(origType);
	}

	/**
	 * @param updated
	 *            the updated to set
	 */
	public void setUpdated(AtomDateConstruct updated) {
		_updated = updated;

		_entry.getItems().remove(origUpdated);
		origUpdated = atomFactory.createUpdated(updated);
		_entry.getItems().add(origUpdated);
	}
}
